#include <iostream>
#include <cstring>

#include <libgen.h>
#include <gtk/gtk.h>
#include <getopt.h>

#ifdef WIN32
#include <w32api.h>
#define _WIN32_IE IE5
#define _WIN32_WINNT Windows2000
#include <shlobj.h>
#endif

#include "support/debug.h"
#include "support/rwmutex.h"
#include "support/thread.h"
#include "support/pathsupport.h"
#include "support/dirtreewalker.h"
#include "support/util.h"

#include "imageutils/tiffsave.h"
#include "imagesource/imagesource_util.h"
#include "imagesource/imagesource_promote.h"
#include "imagesource/imagesource_cms.h"
#include "imagesource/imagesource_montage.h"
#include "imagesource/imagesource_gdkpixbuf.h"
#include "imagesource/pixbuf_from_imagesource.h"

#include "miscwidgets/pixbufview.h"
#include "miscwidgets/colorantselector.h"
#include "miscwidgets/imageselector.h"
#include "miscwidgets/generaldialogs.h"
#include "miscwidgets/errordialogqueue.h"
#include "miscwidgets/uitab.h"
#include "miscwidgets/simplecombo.h"
#include "miscwidgets/pixbuf_from_imagedata.h"

#include "pixbufthumbnail/egg-pixbuf-thumbnail.h"

#include "progressbar.h"

#include "cachedimage.h"

#include "conversionopts.h"
#include "conversionoptsdialog.h"
#include "devicelinkdialog.h"

#include "dialogs.h"

#include "config.h"
#include "gettext.h"
#define _(x) gettext(x)
#define N_(x) gettext_noop(x)

#define PRESET_OTHER_ESCAPE "<other>"

using namespace std;


class PreloadProfiles : public Job
{
	public:
	PreloadProfiles(ProfileManager &pm) : Job("Finding ICC profiles..."), pm(pm)
	{
	}
	void Run(Worker *w)
	{
		Debug[TRACE] << "Getting profile list..." << endl;
		try
		{
			PTMutex::Lock lock(pm);
			ProfileInfo *pi=pm.GetFirstProfileInfo();
			while(pi)
			{
				try
				{
					pi->GetColourSpace();	// Trigger loading and caching of profile info
				}
				catch(const char *err)
				{
					Debug[WARN] << "Error: " << err << std::endl;
				}
				pi=pi->Next();
			}		
		}
		catch(const char *err)
		{
			Debug[ERROR] << "Error: " << err << std::endl;
		}
	}
	protected:
	ProfileManager &pm;
};


int main(int argc,char *argv[])
{
	Debug.SetLevel(TRACE);
	try
	{
#if 1
		JobDispatcher dispatcher(1);
		ConfigFile conf;
		ProfileManager pm(&conf,"[ColourManagement]");
		conf.ParseConfigFile("$HOME/.config/cmyktool/cmyktool.conf");
		dispatcher.AddJob(new PreloadProfiles(pm));
#else
		ConfigFile conf;
		ProfileManager pm(&conf,"[ColourManagement]");
			PTMutex::Lock lock(pm);
			ProfileInfo *pi=pm.GetFirstProfileInfo();
			while(pi)
			{
				try
				{
					pi->GetColourSpace();	// Trigger loading and caching of profile info
				}
				catch(const char *err)
				{
					Debug[WARN] << "Error: " << err << std::endl;
				}
				pi=pi->Next();
			}		
#endif
	}
	catch(const char *err)
	{
		cerr << "Error: " << err << endl;
	}
	return(0);
}

